<?php

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\BubbleableMetadata;
use Drupal\Core\Url;
use Drupal\druhels\TaxonomyHelper;
use Drupal\improvements\ImprovementsHelper;
use Drupal\taxonomy\TermInterface;

/**
 * Implements hook_entity_base_field_info().
 */
function improvements_taxonomy_entity_base_field_info(EntityTypeInterface $entity_type): array {
  if ($entity_type->id() == 'taxonomy_term') {
    $fields = [];

    $fields['stored_depth'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('Depth'))
      ->setSetting('unsigned', TRUE);

    $fields['has_children'] = BaseFieldDefinition::create('boolean')
      ->setLabel(t('Has chilren'))
      ->setDefaultValue(FALSE);

    return $fields;
  }

  return [];
}

/**
 * Implements hook_entity_base_field_info_alter().
 *
 * @param BaseFieldDefinition[] $fields
 *
 * @see improvements_entity_base_field_info_alter()
 */
function improvements_taxonomy_entity_base_field_info_alter(array &$fields, EntityTypeInterface $entity_type): void {
  if ($entity_type->id() == 'taxonomy_term') {
    $fields['name']->setDisplayConfigurable('view', TRUE);
  }
}

/**
 * Implements hook_ENTITY_TYPE_presave(): taxonomy_term.
 */
function improvements_taxonomy_term_presave(TermInterface $term): void {
  // Set "stored_depth" field
  $term->set('stored_depth', TaxonomyHelper::getTermDepth($term));

  // Set "has_chilren" field
  if ($term->get('has_children')->isEmpty()) {
    $term->set('has_children', TaxonomyHelper::termHasChilds($term->id()));
  }
}

/**
 * Implements hook_ENTITY_TYPE_insert(): taxonomy_term.
 */
function improvements_taxonomy_term_insert(TermInterface $term): void {
  // Set parent term "has_children" field value
  $parent_term = $term->get('parent')->entity; /** @var TermInterface $parent_term */
  if ($parent_term && !$parent_term->get('has_children')->value) {
    $parent_term->set('has_children', TRUE)->save();
  }
}

/**
 * Implements hook_ENTITY_TYPE_update(): taxonomy_term.
 */
function improvements_taxonomy_term_update(TermInterface $term): void {
  /** @noinspection All */
  $original_term = $term->original; /** @var TermInterface $original_term */

  // Manualy save children terms to run improvements_taxonomy_term_presave() and update "stored_depth" field
  $term_depth_is_changed = ($term->get('stored_depth')->value != $original_term->get('stored_depth')->value);
  if ($term_depth_is_changed || $original_term->get('stored_depth')->isEmpty()) {
    if ($term_direct_children = TaxonomyHelper::getDirectChildTerms($term->bundle(), $term->id(), TRUE)) {
      foreach ($term_direct_children as $children_term) {
        $children_term->save();
      }
    }
  }

  // Update parent terms "has_children" field
  $term_parent_items = $term->get('parent');
  $original_term_parent_items = $original_term->get('parent');
  $term_parent_is_changed = $term_parent_items->hasAffectingChanges($original_term_parent_items, $term_parent_items->getLangcode());

  if ($term_parent_is_changed) {
    // Update new parent terms
    if ($term_parent_items->entity) {
      foreach ($term_parent_items as $term_parent_item) {
        $parent_term = $term_parent_item->entity; /** @var TermInterface $parent_term */
        if (!$parent_term->get('has_children')->value) {
          $parent_term->set('has_children', TRUE)->save();
        }
      }
    }

    // Update old parent terms
    if ($original_term_parent_items->entity) {
      foreach ($original_term_parent_items as $original_term_parent_item) {
        $original_parent_term = $original_term_parent_item->entity; /** @var TermInterface $original_parent_term */
        $original_parent_term_has_children_new = TaxonomyHelper::termHasChilds($original_parent_term->id());
        if ($original_parent_term->get('has_children')->value != $original_parent_term_has_children_new) {
          $original_parent_term->set('has_children', $original_parent_term_has_children_new)->save();
        }
      }
    }
  }
}

/**
 * Implements hook_ENTITY_TYPE_delete(): taxonomy_term.
 */
function improvements_taxonomy_term_delete(TermInterface $term): void {
  // Update parent term "has_children" field value
  $parent_term = $term->get('parent')->entity; /** @var TermInterface $parent_term */
  if ($parent_term && !TaxonomyHelper::termHasChilds($parent_term->id(), TRUE)) {
    $parent_term->set('has_children', FALSE)->save();
  }
}

/**
 * Implements hook_form_BASE_FORM_ID_alter(): taxonomy_term_form.
 */
function improvements_form_taxonomy_term_form_alter(array &$form, FormStateInterface $form_state, $foo = NULL, $bar = NULL): void {
  $term = $form_state->getFormObject()->getEntity(); /** @var TermInterface $term */

  // Add autofocus to term name field
  if (isset($form['name'])) {
    $form['name']['widget']['0']['value']['#attributes']['autofocus'] = 'autofocus';
  }

  // Change "relations" item type from "details" to "container"
  $form['relations']['#type'] = 'container';

  if (isset($form['relations']['parent'])) {
    // Prepopulate parent field based on $_GET['parent'] value
    // @see improvements_entity_operation()
    if ($parent_id = \Drupal::request()->query->get('parent')) {
      $form['relations']['parent']['#default_value'][0] = $parent_id;
    }

    // Increase "parent" select size
    if (
      $form['relations']['parent']['#type'] == 'select' &&
      $form['relations']['parent']['#multiple'] &&
      count( $form['relations']['parent']['#options']) > 15
    ) {
      $form['relations']['parent']['#size'] = 15;
    }
  }

  // Change text of submit button
  if ($term->isNew() && isset($form['actions']['submit'])) {
    $form['actions']['submit']['#value'] = t('Save and add more');
  }

  // Move "delete" button to end
  if (isset($form['actions']['delete'])) {
    $form['actions']['delete']['#weight'] = 100;
  }
}

/**
 * Implements hook_form_FORM_ID_alter(): taxonomy_overview_terms.
 */
function improvements_form_taxonomy_overview_terms_alter(array &$form, FormStateInterface $form_state): void {
  $form['actions']['reset_alphabetical']['#access'] = FALSE;
}

/**
 * Implements hook_entity_operation().
 */
function improvements_entity_operation(EntityInterface $entity): ?array {
  // Add term operation "Add child term"
  if ($entity->getEntityTypeId() == 'taxonomy_term') {
    $operations = [];

    $url = Url::fromRoute('entity.taxonomy_term.add_form', ['taxonomy_vocabulary' => $entity->bundle()], ['query' => ['parent' => $entity->id()]]);
    $operations['add_child'] = [
      'title' => t('Add child term'),
      'url' => $url,
      'weight' => 10,
    ];

    return $operations;
  }

  return NULL;
}

/**
 * See improvements_entity_type_build().
 */
function improvements_taxonomy_entity_type_build(array &$entity_types): void {
  // See https://www.drupal.org/node/3043840
  if (isset($entity_types['taxonomy_term'])) {
    $entity_types['taxonomy_term']->set('enable_base_field_custom_preprocess_skipping', TRUE);
  }
}

/**
 * Implements hook_token_info().
 */
function improvements_taxonomy_token_info(): array {
  $info = [];

  $info['tokens']['term']['parents_field'] = [
    'name' => t('Parents field value'),
    'description' => t('An array of all the term\'s parents field value, starting with the root.'),
    'type' => 'array',
    'dynamic' => TRUE,
  ];

  return $info;
}

/**
 * Implements hook_tokens().
 */
function improvements_taxonomy_tokens(string $type, array $tokens, array $data, array $options, BubbleableMetadata $bubbleable_metadata): array {
  $replacements = [];

  // [term:*]
  if ($type == 'term' && !empty($data['term'])) {
    $term = $data['term']; /* @var TermInterface $term */
    $token_service = \Drupal::token();

    // [term:parents_field:*]
    if ($parents_field_tokens = $token_service->findWithPrefix($tokens, 'parents_field')) {
      foreach ($parents_field_tokens as $name => $original) {
        [$field_name, ] = explode(':', $name);
        $field_values = ImprovementsHelper::getTermAllParentsFieldValues($term->id(), $field_name);

        if ($array_tokens = $token_service->findWithPrefix($parents_field_tokens, $field_name)) {
          $replacements += $token_service->generate('array', $array_tokens, ['array' => $field_values], $options, $bubbleable_metadata);
        }
        else {
          $replacements[$original] = token_render_array($field_values, $options);
        }
      }
    }
  }

  return $replacements;
}
